#!/usr/bin/env node

// index.ts
import isCI2 from "is-ci";

// helper.ts
import { join } from "path";
import { create } from "logua";
var log = create("squak", "red");
var results = /* @__PURE__ */ new Map();
var cache = (method) => () => {
  if (results.has(method)) {
    return results.get(method);
  }
  const result = method();
  results.set(method, result);
  return result;
};
var getProjectBasePath = () => {
  const currentWorkingDirectory = process.cwd();
  if (currentWorkingDirectory.includes("node_modules") && process.env.INIT_CWD) {
    return process.env.INIT_CWD;
  }
  if (currentWorkingDirectory.includes("node_modules/squak") || currentWorkingDirectory.includes("node_modules\\squak")) {
    return join(currentWorkingDirectory, "../..");
  }
  return currentWorkingDirectory;
};
var hashPath = (options2) => join("node_modules", "squak", options2().hash);
var removeDuplicatePaths = (relativePaths) => {
  const absolutePaths = relativePaths.map((path) => join(getProjectBasePath(), path));
  const noDuplicatesSet = /* @__PURE__ */ new Set();
  const indicesToRemove = [];
  absolutePaths.forEach((path, index) => {
    if (noDuplicatesSet.has(path)) {
      indicesToRemove.push(index);
    }
    noDuplicatesSet.add(path);
  });
  indicesToRemove.reverse();
  indicesToRemove.forEach((index) => {
    relativePaths.splice(index, 1);
  });
  return relativePaths;
};

// configure.ts
import { accessSync, existsSync as existsSync2, constants, readFileSync as readFileSync2, writeFileSync as writeFileSync2, mkdirSync } from "fs";
import { EOL } from "os";
import { join as join5 } from "path";
import formatJson from "pakag";
import merge3 from "deepmerge";
import parse from "parse-gitignore";
import unset from "lodash.unset";
import deepForEach from "deep-for-each";
import isCI from "is-ci";

// options.ts
import { existsSync, writeFileSync, readFileSync } from "fs";
import { join as join2 } from "path";
import glob from "fast-glob";
import merge from "deepmerge";
import { createHash } from "crypto";
var defaultOptions = (pkg) => ({
  entry: [],
  output: "dist",
  test: "test",
  hash: createHash("md5").update(pkg.name).digest("hex").substring(0, 3),
  pkg
});
var options = cache(() => {
  let packageContents;
  try {
    const packageContentsFile = readFileSync(join2(getProjectBasePath(), "package.json"), "utf8");
    packageContents = JSON.parse(packageContentsFile);
  } catch (error) {
    log("Unable to load package.json", "error");
  }
  if (typeof packageContents.name !== "string") {
    log('Missing "name" field in package.json', "error");
  }
  let result = defaultOptions(packageContents);
  if (typeof packageContents.squak === "object") {
    result = merge(result, packageContents.squak, { clone: false });
    if (typeof result.entry === "string") {
      result.entry = [result.entry];
    }
  }
  result.entry = result.entry.concat(["index.ts", "src/index.ts"]);
  result.entry = result.entry.filter((filePath) => existsSync(join2(getProjectBasePath(), filePath)));
  if (result.entry.length === 0) {
    const entryFilePath = join2(getProjectBasePath(), "index.ts");
    writeFileSync(entryFilePath, "// Default entry file created by squak.");
    log(`No entry file found, created one in ${entryFilePath}`);
    result.entry = ["index.ts"];
  }
  result.entry = removeDuplicatePaths(result.entry);
  if (typeof result.test === "string") {
    const hasTests = glob.sync([`${result.test}/**.test.ts`], {
      cwd: getProjectBasePath()
    }).length > 0;
    result.test = hasTests ? result.test : false;
  }
  return result;
});

// configuration/gitignore.ts
var gitignore = () => {
  let entries = ["node_modules", "package-lock.json", "tsconfig.json", options().output];
  const fromPackage = options().gitignore;
  if (fromPackage && Array.isArray(fromPackage) && fromPackage.length > 0) {
    entries = entries.concat(fromPackage);
  }
  return entries;
};

// configuration/package.ts
import { join as join3, sep } from "path";
var packagePropertiesToUpdate = [
  // Keep node version up-to-date.
  "engines",
  "prettier",
  "eslintConfig.extends"
];
var packageJson = () => {
  const pkg = {
    scripts: options().pkg.scripts,
    type: "module",
    prettier: `.${sep}${join3(hashPath(options), ".prettierrc.json")}`,
    eslintConfig: {
      extends: `.${sep}${join3(hashPath(options), ".eslintrc.json")}`
    },
    engines: {
      node: ">= 16"
    }
  };
  if (!options().pkg.scripts) {
    pkg.scripts = {
      start: "squak start",
      production: "squak production"
    };
  }
  if (options().test) {
    pkg.scripts.test = "squak test";
    pkg.jest = {
      transform: {
        "^.+\\.(j|t)sx?$": "@swc/jest"
      }
    };
  }
  return pkg;
};

// configuration/typescript.ts
import merge2 from "deepmerge";
var tsconfig = () => {
  const userTSConfig = {
    extends: `squak/${options().hash}/tsconfig`
  };
  let packageTSConfig = {
    compilerOptions: {
      target: "esnext",
      module: "esnext",
      esModuleInterop: true,
      moduleResolution: "bundler",
      skipLibCheck: true,
      forceConsistentCasingInFileNames: true,
      outDir: `../../../${options().output}`,
      baseUrl: "../../.."
    },
    files: options().entry.map((entry) => `../../../${entry}`),
    exclude: [`../../../${options().output}`]
  };
  if (options().test) {
    packageTSConfig.include = [`../../../${options().test}`];
  }
  if (options().tsconfig) {
    packageTSConfig = merge2(packageTSConfig, options().tsconfig, {
      clone: false
    });
  }
  return [userTSConfig, packageTSConfig];
};

// configuration/eslint.ts
import { join as join4 } from "path";
var customRules = {
  // Use named exports to make it easier to find usages.
  "import/prefer-default-export": 0,
  // Allow assignment to function param properties, like parameter.innerHTML = ...
  "no-param-reassign": [2, { props: false }]
};
var eslint = () => ({
  // base as we don't need React on the server.
  extends: ["airbnb-base", "airbnb-typescript/base", "prettier"],
  rules: customRules,
  ignorePatterns: [options().output],
  env: {
    node: true
  },
  parser: "@typescript-eslint/parser",
  parserOptions: {
    project: join4(getProjectBasePath(), "tsconfig.json")
  },
  overrides: [
    {
      // Tests
      files: ["**/*.test.ts"],
      env: {
        jest: true
      }
    }
  ]
});

// configuration/prettier.ts
var prettier = () => ({
  singleQuote: true,
  semi: false,
  printWidth: 100
});
var prettierIgnore = () => [options().output];

// configure.ts
var ensureHashDirectory = () => {
  const hashDirectoryPath = join5(getProjectBasePath(), hashPath(options));
  if (!existsSync2(hashDirectoryPath)) {
    mkdirSync(hashDirectoryPath, { recursive: true });
  }
};
var configurePrettier = () => {
  const prettierConfigPath = join5(getProjectBasePath(), hashPath(options), `.prettierrc.json`);
  const prettierignorePath = join5(getProjectBasePath(), hashPath(options), `.prettierignore`);
  try {
    ensureHashDirectory();
    writeFileSync2(prettierConfigPath, formatJson(JSON.stringify(prettier()), { sort: false }));
  } catch (_) {
    log(
      `Couldn't write ${prettierConfigPath}, therefore this plugin might not work as expected`,
      "warning"
    );
  }
  try {
    writeFileSync2(prettierignorePath, prettierIgnore().join(EOL));
  } catch (_) {
    log(
      `Couldn't write ${prettierignorePath}, therefore this plugin might not work as expected`,
      "warning"
    );
  }
};
var configureEslint = () => {
  const eslintConfigPath = join5(getProjectBasePath(), hashPath(options), `.eslintrc.json`);
  try {
    ensureHashDirectory();
    writeFileSync2(eslintConfigPath, formatJson(JSON.stringify(eslint()), { sort: false }));
  } catch (_) {
    log(
      `Couldn't write ${eslintConfigPath}, therefore this plugin might not work as expected`,
      "warning"
    );
  }
};
var writeUserAndPackageConfig = (userConfig, packageConfig, userTSConfigPath, packageTSConfigPath) => {
  try {
    ensureHashDirectory();
    writeFileSync2(packageTSConfigPath, formatJson(JSON.stringify(packageConfig), { sort: false }));
  } catch (_) {
    log(
      `Couldn't write ${packageTSConfigPath}, therefore this plugin might not work as expected`,
      "warning"
    );
  }
  try {
    writeFileSync2(userTSConfigPath, formatJson(JSON.stringify(userConfig), { sort: false }));
  } catch (_) {
    log(
      `Couldn't write ${userTSConfigPath}, therefore this plugin might not work as expected`,
      "warning"
    );
  }
};
var adaptConfigToRoot = (packageConfig) => {
  deepForEach(packageConfig, (value, key, subject) => {
    const baseFromPackagePath = "../../../";
    if (typeof value === "string" && value.includes(baseFromPackagePath)) {
      subject[key] = value.replace(baseFromPackagePath, "");
    }
    if (value === "../../..") {
      subject[key] = ".";
    }
  });
};
var writeOnlyUserConfig = (userConfig, packageConfig, userTSConfigPath) => {
  try {
    delete userConfig.extends;
    adaptConfigToRoot(packageConfig);
    writeFileSync2(
      userTSConfigPath,
      formatJson(JSON.stringify(merge3(userConfig, packageConfig, { clone: false })), {
        sort: false
      })
    );
  } catch (_) {
    log(
      `Couldn't write ./tsconfig.json, therefore this plugin might not work as expected`,
      "warning"
    );
  }
};
var configureTsconfig = () => {
  const userTSConfigPath = join5(getProjectBasePath(), "tsconfig.json");
  const packageTSConfigPath = join5(getProjectBasePath(), hashPath(options), `tsconfig.json`);
  const [userConfig, packageConfig] = tsconfig();
  try {
    accessSync(
      packageTSConfigPath,
      // eslint-disable-next-line no-bitwise
      constants.F_OK | constants.R_OK | constants.W_OK
    );
    writeUserAndPackageConfig(userConfig, packageConfig, userTSConfigPath, packageTSConfigPath);
  } catch (_) {
    writeOnlyUserConfig(userConfig, packageConfig, userTSConfigPath);
  }
};
var configureGitignore = () => {
  const gitIgnorePath = join5(getProjectBasePath(), ".gitignore");
  let entries = [];
  if (existsSync2(gitIgnorePath)) {
    entries = entries.concat(parse(readFileSync2(gitIgnorePath, "utf8")).patterns);
  }
  entries = entries.concat(gitignore());
  entries = Array.from(new Set(entries)).concat("");
  writeFileSync2(gitIgnorePath, entries.join("\r\n"));
};
var removePropertiesToUpdate = (pkg) => {
  if (!isCI) {
    packagePropertiesToUpdate.forEach((key) => unset(pkg, key));
  }
};
var configurePackageJson = () => {
  const packageJsonContents = options().pkg;
  let generatedPackageJson = packageJson();
  removePropertiesToUpdate(packageJsonContents);
  generatedPackageJson = merge3(generatedPackageJson, packageJsonContents, {
    clone: false
  });
  writeFileSync2(
    join5(getProjectBasePath(), "./package.json"),
    formatJson(JSON.stringify(generatedPackageJson))
  );
  options().pkg = generatedPackageJson;
};
var configure = () => {
  configurePackageJson();
  configureGitignore();
  configureTsconfig();
  configureEslint();
  configurePrettier();
};

// script/build.ts
import { existsSync as existsSync3, rmSync } from "fs";
import { join as join6 } from "path";
import { execSync } from "child_process";
import esbuild from "esbuild";

// configuration/esbuild.ts
import merge4 from "deepmerge";
var esbuildConfiguration = () => {
  const userDependencies = [].concat(Object.keys(options().pkg.dependencies || {})).concat(Object.keys(options().pkg.peerDependencies || {}));
  const userESBuildConfiguration = typeof options().esbuild === "object" ? options().esbuild : {};
  let buildOptions = {
    // entryPoints needs to be an array.
    entryPoints: options().entry,
    outdir: options().output,
    minify: false,
    bundle: true,
    external: userDependencies,
    sourcemap: false,
    color: true,
    absWorkingDir: process.cwd(),
    target: "node14",
    platform: "node",
    format: "esm",
    tsconfig: "tsconfig.json"
  };
  buildOptions = merge4(buildOptions, userESBuildConfiguration);
  return buildOptions;
};

// script/build.ts
var javaScriptBuild = async (watch) => {
  let firstBuild = true;
  const onWatchPlugin = {
    name: "on-watch-plugin",
    setup(build2) {
      build2.onStart(() => {
        if (!firstBuild) {
          log("rebuilding...");
        } else {
          firstBuild = false;
        }
      });
    }
  };
  const buildOptions = esbuildConfiguration();
  let context;
  try {
    context = await esbuild.context({ ...buildOptions, plugins: [onWatchPlugin] });
    if (!watch) {
      await context.rebuild();
    }
  } catch (error) {
    process.exit(1);
  }
  if (watch) {
    await context.watch();
  }
  if (!watch) {
    await context.dispose();
  }
  return () => context.dispose();
};
var build = (watch = false) => {
  if (!watch) {
    log("Checking types...");
    const tsconfigPath = join6(process.cwd(), "tsconfig.json");
    execSync(`tsc --project "${tsconfigPath}" --noEmit`, { stdio: "inherit", cwd: process.cwd() });
  }
  log(watch ? "watching..." : "building...");
  const outputDirectory = join6(process.cwd(), options().output);
  if (!watch) {
    if (existsSync3(outputDirectory)) {
      rmSync(outputDirectory, { recursive: true });
    }
  }
  return javaScriptBuild(watch);
};

// script/lint.ts
import { ESLint } from "eslint";
import { execSync as execSync2 } from "child_process";
var lint = async () => {
  log("formatting files...");
  const configurationFolder = hashPath(options);
  execSync2(
    `prettier --write "**/*.ts" --config "${configurationFolder}/.prettierrc.json" --ignore-path "${configurationFolder}/.prettierignore"`,
    { stdio: "inherit", cwd: process.cwd() }
  );
  console.log("");
  log("linting files...");
  const linter = new ESLint({
    fix: true,
    extensions: [".ts"]
  });
  const results2 = await linter.lintFiles(".");
  await ESLint.outputFixes(results2);
  const formatter = await linter.loadFormatter("stylish");
  const resultText = formatter.format(results2);
  if (resultText) {
    console.log(resultText);
  }
  return results2;
};

// script/start.ts
import { spawn } from "child_process";
var start = async () => {
  const close = await build(true);
  const instances = options().entry.map((entry) => {
    const entryFile = `${options().output}/${entry.replace(".ts", ".js")}`;
    return spawn("nodemon", ["-q", entryFile], {
      stdio: "inherit",
      cwd: process.cwd(),
      shell: true
    });
  });
  return async () => {
    await close();
    instances.forEach((instance) => instance.kill());
  };
};

// script/test.ts
import { execSync as execSync3 } from "child_process";
var test = () => {
  if (!options().test) {
    log(`No tests found in ${options().test}`, "warning");
  }
  const additionalArguments = process.argv.slice(3);
  log("running tests...");
  execSync3(`jest ${additionalArguments.join(" ")}`, {
    stdio: "inherit",
    cwd: process.cwd()
  });
};

// script/production.ts
import { spawn as spawn2, execSync as execSync4 } from "child_process";
var production = async () => {
  await build(false);
  log("Pruning devDependencies...");
  execSync4("npm prune --production --legacy-peer-deps", { stdio: "inherit", cwd: process.cwd() });
  log("Starting server...");
  const instances = options().entry.map((entry) => {
    const entryFile = `${options().output}/${entry.replace(".ts", ".js")}`;
    return spawn2("node", [entryFile], {
      stdio: "inherit",
      cwd: process.cwd(),
      shell: true
    });
  });
  return () => instances.forEach((instance) => instance.kill());
};

// index.ts
var scripts = {
  start,
  test,
  lint,
  production,
  build
};
var script = process.argv.slice(2)[0];
if (!Object.keys(scripts).includes(script)) {
  log("Please provide a valid script", "error");
}
configure();
try {
  scripts[script]();
} catch (error) {
  log(`script ${script} exited with an error`);
  if (script !== "test" && !isCI2) {
    console.error(error);
  }
  if (isCI2) {
    throw new Error(error);
  }
}
//# sourceMappingURL=index.js.map
